SystemOrganization addCategory: #'Pier-Squeak-Persistency'!

PRPersistency subclass: #PRImagePersistency
	instanceVariableNames: 'duration history process savedImages maxSavedImages'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Pier-Squeak-Persistency'!

!PRImagePersistency methodsFor: 'private' stamp: 'lr 11/20/2007 16:00'!
add: aContext
	history addLast: aContext.
	[ history size < 100 ]
		whileFalse: [ history removeFirst ]! !

!PRImagePersistency methodsFor: 'private' stamp: 'tg 11/23/2007 00:46'!
deleteOldestImage
	FileDirectory default 
		deleteFileNamed: (self savedImages first, '.', FileDirectory imageSuffix)
		ifAbsent: [].
	FileDirectory default 
		deleteFileNamed: (self savedImages first, '.', FileDirectory changeSuffix)
		ifAbsent: [].
	savedImages removeFirst.! !

!PRImagePersistency methodsFor: 'enumerating' stamp: 'lr 11/20/2007 15:39'!
do: aBlock
	history do: aBlock! !

!PRImagePersistency methodsFor: 'accessing' stamp: 'lr 9/29/2006 08:39'!
duration
	^ duration! !

!PRImagePersistency methodsFor: 'accessing' stamp: 'lr 9/29/2006 08:39'!
duration: aDuration
	duration := aDuration! !

!PRImagePersistency methodsFor: 'actions' stamp: 'lr 11/20/2007 16:00'!
execute: aCommand
	self mutex critical: [
		self apply: aCommand.
		aCommand isLogged ifTrue: [
			self add: aCommand context.
			self log: aCommand context ] ].
	self start! !

!PRImagePersistency methodsFor: 'initialization' stamp: 'tg 11/23/2007 00:47'!
initialize
	super initialize.
	duration := 1 hour.
	savedImages := OrderedCollection new. 
	maxSavedImages := SmallInteger maxVal.! !

!PRImagePersistency methodsFor: 'testing' stamp: 'lr 1/17/2007 11:44'!
isRunning
	^ process notNil and: [ process isTerminated not ]! !

!PRImagePersistency methodsFor: 'private' stamp: 'lr 1/17/2007 10:37'!
log: aContext
	| stream |
	stream := FileDirectory default
		fileNamed: 'transactions.txt'.
	[ self log: aContext on: stream setToEnd ]
		ensure: [ stream close ]! !

!PRImagePersistency methodsFor: 'private' stamp: 'lr 2/12/2007 19:07'!
log: aContext on: aStream
	aStream nextPutAll: ' == '; nextPutAll: aContext command class label; nextPutAll: ' == '; cr.
	aStream nextPutAll: ' -- context -- '; cr.
	aContext description do: [ :each |
		aStream
			nextPutAll: each label; nextPutAll: ': ';
			nextPutAll: (each toString: (aContext
				readUsing: each)); cr ].
	aStream nextPutAll: ' -- command -- '; cr.
	aContext command description do: [ :each |
		aStream
			nextPutAll: each label; nextPutAll: ': ';
			nextPutAll: (each toString: (aContext command
				readUsing: each)); cr ].
	aStream cr; cr! !

!PRImagePersistency methodsFor: 'accessing' stamp: 'tg 11/23/2007 00:14'!
maxSavedImages
	^maxSavedImages! !

!PRImagePersistency methodsFor: 'accessing' stamp: 'tg 11/23/2007 00:15'!
maxSavedImages: anInteger
	maxSavedImages := anInteger! !

!PRImagePersistency methodsFor: 'private' stamp: 'lr 10/16/2006 11:33'!
process
	(Delay forDuration: self duration)
		wait.
	process := nil.
	self snapshot! !

!PRImagePersistency methodsFor: 'enumerating' stamp: 'lr 11/20/2007 15:39'!
reverseDo: aBlock
	history reverseDo: aBlock! !

!PRImagePersistency methodsFor: 'private' stamp: 'tg 1/14/2008 23:34'!
saveImageAs: aString
	| image |
	image := SmalltalkImage current.
	image closeSourceFiles.
	FileDirectory default 
		copyFileWithoutOverwriteConfirmationNamed: SmalltalkImage current changesName 
		toFileNamed: (
				FileDirectory default pathName,
				FileDirectory slash, aString, '.', 
				FileDirectory changeSuffix asString).
	Smalltalk setMacFileInfoOn: (
				FileDirectory default pathName, 
				FileDirectory slash, aString, '.', 
				FileDirectory changeSuffix asString).
	image changeImageNameTo: (
				FileDirectory default pathName,
				FileDirectory slash, aString, '.',
				FileDirectory imageSuffix asString).
	image 
		openSourceFiles;  "so SNAPSHOT appears in new changes file"
		saveImageSegments;
		snapshot: true andQuit: false.

	self savedImages addLast: aString.
	self savedImages size > self maxSavedImages ifTrue: [self deleteOldestImage]! !

!PRImagePersistency methodsFor: 'accessing' stamp: 'tg 11/23/2007 00:15'!
savedImages
	^savedImages! !

!PRImagePersistency methodsFor: 'initialization' stamp: 'lr 11/20/2007 15:36'!
setKernel: aKernel
	super setKernel: aKernel.
	self stop.
	history := OrderedCollection new! !

!PRImagePersistency methodsFor: 'actions' stamp: 'tg 11/23/2007 00:19'!
snapshot
	"Save the whole image for now."

	| filename |
	filename := DateAndTime now asSortableString.
	Smalltalk at: #UnixProcess ifPresent: [ :class |
		(class thisOSProcess pid notNil
			and: [ class respondsTo: #saveImageInBackground:nice: ]) ifTrue: [
				Smalltalk at: #WARegistry ifPresent: [ :registry | registry clearAllHandlers ].
				^ class saveImageInBackground: filename nice: true ] ].
	self saveImageAs: filename.! !

!PRImagePersistency methodsFor: 'process' stamp: 'lr 2/12/2007 19:07'!
start
	self isRunning
		ifTrue: [ ^ self ].
	process := [ self process ]
		forkAt: 30 named: 'pier'! !

!PRImagePersistency methodsFor: 'process' stamp: 'lr 2/12/2007 19:07'!
stop
	self isRunning
		ifTrue: [ process terminate ].
	process := nil! !

PRPersistencyTest subclass: #PRImagePersistencyTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Pier-Squeak-Persistency'!

!PRImagePersistencyTest class methodsFor: 'testing' stamp: 'lr 3/30/2007 22:22'!
isAbstract
	^ false! !

!PRImagePersistencyTest methodsFor: 'private' stamp: 'lr 3/30/2007 22:22'!
actualClass
	^ PRImagePersistency! !
